@@ -22,14 +22,14 @@ def loadCSSLexer() -> lexers.CSSLexer:
     return CSSLexer()
 
 
-customLexers: t.Dict[str, pygments.lexer.Lexer] = {"css": loadCSSLexer}
+customLexers: dict[str, pygments.lexer.Lexer] = {"css": loadCSSLexer}
 
 
 @dataclasses.dataclass
 class ColoredText:
     text: str
     # None indicates uncolored text
-    color: t.Optional[str]
+    color: str | None
 
 
 def addSyntaxHighlighting(doc: t.SpecT) -> None:
@@ -48,12 +48,12 @@ def addSyntaxHighlighting(doc: t.SpecT) -> None:
             # Element was already highlighted, but needs styles
             highlightingOccurred = True
         elif lang:
-            highlightEl(el, lang)
+            highlightEl(doc, el, lang)
             highlightingOccurred = True
         # Find whether to add line numbers
         ln = determineLineNumbers(doc, el)
         if ln.addNumbers or ln.highlights:
-            addLineWrappers(el, ln)
+            addLineWrappers(doc, el, ln)
             if ln.addNumbers:
                 lineWrappingOccurred = True
             if ln.highlights:
@@ -70,7 +70,7 @@ def addSyntaxHighlighting(doc: t.SpecT) -> None:
         doc.extraStyles["style-darkmode"] += getLineHighlightingDarkmodeStyles()
 
 
-def determineHighlightLang(doc: t.SpecT, el: t.ElementT) -> t.Optional[t.Union[str, t.Literal[False]]]:
+def determineHighlightLang(doc: t.SpecT, el: t.ElementT) -> str | t.Literal[False] | None:
     # Either returns a normalized highlight lang,
     # False indicating the element was already highlighted,
     # or None indicating the element shouldn't be highlighted.
@@ -85,7 +85,7 @@ def determineHighlightLang(doc: t.SpecT, el: t.ElementT) -> t.Optional[t.Union[s
     if attr == "highlight":
         return lang
     # Highlight-by-default, if applicable.
-    if el.tag in ["pre", "xmp"] and h.hasClass(el, "idl"):
+    if el.tag in ["pre", "xmp"] and h.hasClass(doc, el, "idl"):
         return "webidl"
     return doc.md.defaultHighlight
 
@@ -98,7 +98,7 @@ class LineNumberOptions:
     startingLine: int
     # Which lines, if any, to specially highlight
     # (relative to the starting line)
-    highlights: t.Set[int]
+    highlights: set[int]
 
 
 def determineLineNumbers(doc: t.SpecT, el: t.ElementT) -> LineNumberOptions:
@@ -151,19 +151,19 @@ def determineLineNumbers(doc: t.SpecT, el: t.ElementT) -> LineNumberOptions:
     return LineNumberOptions(addLineNumbers, lineStart, lineHighlights)
 
 
-def highlightEl(el: t.ElementT, lang: str) -> None:
+def highlightEl(doc: t.SpecT, el: t.ElementT, lang: str) -> None:
     text = h.textContent(el)
-    coloredText: t.Optional[t.Deque[ColoredText]]
+    coloredText: t.Deque[ColoredText] | None
     if lang == "webidl":
         coloredText = highlightWithWebIDL(text, el=el)
     else:
         coloredText = highlightWithPygments(text, lang, el=el)
     if coloredText is not None:
         mergeHighlighting(el, coloredText)
-        h.addClass(el, "highlight")
+        h.addClass(doc, el, "highlight")
 
 
-def highlightWithWebIDL(text: str, el: t.ElementT) -> t.Optional[t.Deque[ColoredText]]:
+def highlightWithWebIDL(text: str, el: t.ElementT) -> t.Deque[ColoredText] | None:
     """
     Trick the widlparser emitter,
     which wants to output HTML via wrapping with start/end tags,
@@ -188,22 +188,20 @@ def highlightWithWebIDL(text: str, el: t.ElementT) -> t.Optional[t.Deque[Colored
 
         def markup_type_name(
             self, text: str, construct: widlparser.constructs.Construct
-        ) -> t.Tuple[t.Optional[str], t.Optional[str]]:
+        ) -> tuple[str | None, str | None]:
             return ("\1n\2", "\3")
 
-        def markup_name(
-            self, text: str, construct: widlparser.constructs.Construct
-        ) -> t.Tuple[t.Optional[str], t.Optional[str]]:
+        def markup_name(self, text: str, construct: widlparser.constructs.Construct) -> tuple[str | None, str | None]:
             return ("\1g\2", "\3")
 
         def markup_keyword(
             self, text: str, construct: widlparser.constructs.Construct
-        ) -> t.Tuple[t.Optional[str], t.Optional[str]]:
+        ) -> tuple[str | None, str | None]:
             return ("\1b\2", "\3")
 
         def markup_enum_value(
             self, text: str, construct: widlparser.constructs.Construct
-        ) -> t.Tuple[t.Optional[str], t.Optional[str]]:
+        ) -> tuple[str | None, str | None]:
             return ("\1s\2", "\3")
 
     if "\1" in text or "\2" in text or "\3" in text:
@@ -219,7 +217,7 @@ def highlightWithWebIDL(text: str, el: t.ElementT) -> t.Optional[t.Deque[Colored
 
 def coloredTextFromWidlStack(widlText: str) -> t.Deque[ColoredText]:
     coloredTexts: t.Deque[ColoredText] = collections.deque()
-    colors: t.List[str] = []
+    colors: list[str] = []
     currentText = ""
     mode = "text"
     for char in widlText:
@@ -262,7 +260,7 @@ def coloredTextFromWidlStack(widlText: str) -> t.Deque[ColoredText]:
     return coloredTexts
 
 
-def highlightWithPygments(text: str, lang: str, el: t.ElementT) -> t.Optional[t.Deque[ColoredText]]:
+def highlightWithPygments(text: str, lang: str, el: t.ElementT) -> t.Deque[ColoredText] | None:
     import pygments  # pylint: disable=redefined-outer-name
     from pygments.formatters.other import RawTokenFormatter
 
@@ -297,11 +295,12 @@ def mergeHighlighting(el: t.ElementT, coloredText: t.Sequence[ColoredText]) -> N
             if h.isElement(node):
                 h.appendChild(el, colorizeEl(node, coloredText))
             else:
+                assert isinstance(node, str)
                 h.appendChild(el, *colorizeText(node, coloredText), allowEmpty=True)
         return el
 
-    def colorizeText(text: str, coloredText: t.Deque[ColoredText]) -> t.List[t.NodeT]:
-        nodes: t.List[t.NodeT] = []
+    def colorizeText(text: str, coloredText: t.Deque[ColoredText]) -> list[t.NodeT]:
+        nodes: list[t.NodeT] = []
         while text and coloredText:
             nextColor = coloredText.popleft()
             if len(nextColor.text) <= len(text):
@@ -396,7 +395,7 @@ def coloredTextFromRawTokens(text: str) -> t.Deque[ColoredText]:
             list.append(ct)
 
     textList: t.Deque[ColoredText] = collections.deque()
-    currentCT: t.Optional[ColoredText] = None
+    currentCT: ColoredText | None = None
     for line in text.split("\n"):
         if not line:
             continue
@@ -418,7 +417,7 @@ def coloredTextFromRawTokens(text: str) -> t.Deque[ColoredText]:
     return textList
 
 
-def normalizeLanguageName(lang: str) -> str:
+def normalizeLanguageName(lang: str | None) -> str | None:
     # Translates some names to ones Pygment understands
     if lang == "aspnet":
         return "aspx-cs"
@@ -437,7 +436,7 @@ def normalizeHighlightMarkers(doc: t.SpecT) -> None:
             el.set("highlight", match.group(1))
 
 
-def lexerFromLang(lang: str) -> t.Optional[pygments.lexer.Lexer]:
+def lexerFromLang(lang: str) -> pygments.lexer.Lexer | None:
     if lang in customLexers:
         return customLexers[lang]()
     try:
@@ -449,7 +448,7 @@ def lexerFromLang(lang: str) -> t.Optional[pygments.lexer.Lexer]:
         return None
 
 
-def addLineWrappers(el: t.ElementT, options: LineNumberOptions) -> t.ElementT:
+def addLineWrappers(doc: t.SpecT, el: t.ElementT, options: LineNumberOptions) -> t.ElementT:
     # Wrap everything between each top-level newline with a line tag.
     # Add an attr for the line number, and if needed, the end line.
     lineWrapper = h.E.span({"class": "line"})
@@ -457,6 +456,7 @@ def addLineWrappers(el: t.ElementT, options: LineNumberOptions) -> t.ElementT:
         if h.isElement(node):
             h.appendChild(lineWrapper, node)
         else:
+            assert isinstance(node, str)
             while True:
                 if "\n" in node:
                     pre, _, post = node.partition("\n")
@@ -465,6 +465,7 @@ def addLineWrappers(el: t.ElementT, options: LineNumberOptions) -> t.ElementT:
                     h.appendChild(el, lineWrapper)
                     lineWrapper = h.E.span({"class": "line"})
                     node = post
+                    assert isinstance(node, str)
                 else:
                     h.appendChild(lineWrapper, node)
                     break
@@ -473,26 +474,24 @@ def addLineWrappers(el: t.ElementT, options: LineNumberOptions) -> t.ElementT:
         h.appendChild(el, lineWrapper)
     # Number the lines
     lineNumber = options.startingLine
-    for lineNo, node in grouper(h.childNodes(el), 2):
-        assert lineNo is not None
-        assert node is not None
+    for lineNo, node in t.cast("tuple[t.ElementT, t.ElementT]", grouper(h.childNodes(el), 2)):
         if options.addNumbers or lineNumber in options.highlights:
             lineNo.set("data-line", str(lineNumber))
         if lineNumber in options.highlights:
-            h.addClass(node, "highlight-line")
-            h.addClass(lineNo, "highlight-line")
+            h.addClass(doc, node, "highlight-line")
+            h.addClass(doc, lineNo, "highlight-line")
         internalNewlines = countInternalNewlines(node)
         if internalNewlines:
             for i in range(1, internalNewlines + 1):
                 if (lineNumber + i) in options.highlights:
-                    h.addClass(lineNo, "highlight-line")
-                    h.addClass(node, "highlight-line")
+                    h.addClass(doc, lineNo, "highlight-line")
+                    h.addClass(doc, node, "highlight-line")
                     lineNo.set("data-line", str(lineNumber))
             lineNumber += internalNewlines
             if options.addNumbers:
                 lineNo.set("data-line-end", str(lineNumber))
         lineNumber += 1
-    h.addClass(el, "line-numbered")
+    h.addClass(doc, el, "line-numbered")
     return el
 
 
@@ -502,6 +501,7 @@ def countInternalNewlines(el: t.ElementT) -> int:
         if h.isElement(node):
             count += countInternalNewlines(node)
         else:
+            assert isinstance(node, str)
             count += node.count("\n")
     return count
 
@@ -751,9 +751,7 @@ def getLineHighlightingDarkmodeStyles() -> str:
 """
 
 
-def grouper(
-    iterable: t.Sequence[T], n: int, fillvalue: t.Optional[T] = None
-) -> itertools.zip_longest[t.Tuple[t.Optional[T], ...]]:
+def grouper(iterable: t.Sequence[T], n: int, fillvalue: T | None = None) -> itertools.zip_longest[tuple[T | None, ...]]:
     "Collect data into fixed-length chunks or blocks"
     # grouper('ABCDEFG', 3, 'x') --> ABC DEF Gxx
     args = [iter(iterable)] * n
