@@ -109,7 +109,7 @@ def parseRangeComponent(val: str) -> tuple[str | None, float | int]:
     num: float | int
     val = val.strip()
     if val[0] in ["-", "−"]:
-        sign = "−"
+        sign = "-"
         signVal = -1
         val = val[1:]
 
@@ -122,10 +122,10 @@ def parseRangeComponent(val: str) -> tuple[str | None, float | int]:
     if match is None:
         return None, 0
     (digits, unit) = match.groups()
-    num = int(digits)
+    num = int(digits) * signVal
     val = str(num)
 
-    return sign + val + unit, num
+    return val + unit, num
 
 
 def transformMaybePlaceholders(doc: t.SpecT) -> None:
@@ -678,18 +678,28 @@ def varReplacer(match: re.Match) -> t.NodeT:
 
 inlineLinkRe = re.compile(
     r"""
-                            (\\)?
-                            \[([^\]]*)\]
-                            \(\s*
-                            ([^\s)]+)
-                            \s*(?:"([^"]*)")?\s*
-                            \)""",
+                    (\\)?
+                    \[([^\]]+)\]
+                    \(\s*
+                    (
+                        (?:\\[()]|[^\s"()])*
+                        (?:
+                            # Optional top-level paren group
+                            (?:\((?:\\[()]|[^\s"()])*\))
+                            (?:\\[()]|[^\s"()])*
+                        )*
+                    )
+                    \s*(?:"([^"]*)")?\s*
+                    \)""",
     re.X,
 )
 
 
 def inlineLinkReplacer(match: re.Match) -> t.NodeT:
     _, text, href, title = match.groups()
+    # Remove escapes from parens.
+    href = re.sub(r"\\\(", "(", href)
+    href = re.sub(r"\\\)", ")", href)
     if title:
         attrs = {"href": href, "title": title}
     else:
