@@ -5,11 +5,9 @@ import os
 import re
 from collections import defaultdict
 
-import certifi
 import tenacity
 import requests
 from alive_progress import alive_it
-from json_home_client import Client as APIClient
 
 from .. import config, messages as m, t
 
@@ -24,43 +22,7 @@ if t.TYPE_CHECKING:
     MethodsT: t.TypeAlias = defaultdict[str, dict[str, t.Any]]
     ForsT: t.TypeAlias = defaultdict[str, list[str]]
 
-    class HeadingT(t.TypedDict):
-        url: str
-        number: str
-        text: str
-        spec: str
-
-    ShepherdAnchorT = t.TypedDict(
-        "ShepherdAnchorT",
-        {
-            "name": t.Required[str],
-            "type": t.Required[str],
-            "for": list[str],
-            "section": bool,
-            "title": str,
-            "status": t.Required[str],
-            "normative": bool,
-            "export": bool,
-            "linking_text": list[str],
-            "uri": t.Required[str],
-        },
-        total=False,
-    )
-
-    WebrefAnchorT = t.TypedDict(
-        "WebrefAnchorT",
-        {
-            "id": t.Required[str],
-            "type": t.Required[str],
-            "for": list[str],
-            "informative": bool,
-            "access": t.Required[str],
-            "linkingText": list[str],
-            "href": t.Required[str],
-        },
-        total=False,
-    )
-
+    # https://github.com/w3c/reffy/blob/main/schemas/browserlib/extract-headings.json
     class WebrefHeadingT(t.TypedDict, total=False):
         id: t.Required[str]
         href: t.Required[str]
@@ -68,8 +30,26 @@ if t.TYPE_CHECKING:
         level: t.Required[int]
         number: str
 
-    # Need to use function form due to "for" key
-    # being invalid as a property name
+    class HeadingT(t.TypedDict):
+        url: str
+        number: str
+        text: str
+        spec: str
+
+    # https://github.com/w3c/reffy/blob/main/schemas/browserlib/extract-dfns.json
+    WebrefAnchorT = t.TypedDict(
+        "WebrefAnchorT",
+        {
+            "href": t.Required[str],
+            "linkingText": list[str],
+            "type": t.Required[str],
+            "for": t.Required[list[str]],
+            "access": t.Required[str],
+            "informative": t.Required[bool],
+        },
+        total=False,
+    )
+
     AnchorT = t.TypedDict(
         "AnchorT",
         {
@@ -85,16 +65,6 @@ if t.TYPE_CHECKING:
         },
     )
 
-    class ShepherdSpecT(t.TypedDict, total=False):
-        name: t.Required[str]
-        short_name: t.Required[str]
-        title: t.Required[str]
-        level: t.Required[int]
-        description: str | None
-        base_uri: str | None
-        draft_uri: str | None
-        abstract: str | None
-
     class WebrefSpecT(t.TypedDict, total=False):
         url: t.Required[str]
         shortname: t.Required[str]
@@ -104,6 +74,7 @@ if t.TYPE_CHECKING:
         nightly: t.Required[dict[str, str | list[str]]]
         release: dict[str, str | list[str]]
         seriesVersion: str
+        seriesComposition: str
         links: str
         refs: str
         idl: str
@@ -122,17 +93,12 @@ if t.TYPE_CHECKING:
         level: int | None
 
 
-def progressMessager(index: int, total: int) -> t.Callable[[], None]:
-    return lambda: m.say(f"Downloading data for spec {index}/{total}...")
-
-
 def update(path: str, dryRun: bool = False) -> set[str] | None:
     specs: SpecsT = dict()
     anchors: AnchorsT = defaultdict(list)
     headings: AllHeadingsT = {}
 
     gatherWebrefData(specs, anchors, headings)
-    gatherShepherdData(specs, anchors, headings)
     cleanSpecHeadings(headings)
     methods = extractMethodData(anchors)
     fors = extractForsData(anchors)
@@ -189,7 +155,7 @@ def gatherWebrefData(specs: SpecsT, anchors: AnchorsT, headings: AllHeadingsT) -
 
     progress = alive_it(currentWebrefData, dual_line=True)
     for rawWSpec in t.cast("t.Generator[WebrefSpecT, None, None]", progress):
-        spec = genWebrefSpec(rawWSpec)
+        spec = canonSpecFromWebref(rawWSpec)
         currentUrl = spec["current_url"]
         assert currentUrl is not None
         progress.text(spec["vshortname"].lower())
@@ -197,16 +163,17 @@ def gatherWebrefData(specs: SpecsT, anchors: AnchorsT, headings: AllHeadingsT) -
         specs[spec["vshortname"].lower()] = spec
         specHeadings: HeadingsT = {}
         headings[spec["vshortname"]] = specHeadings
-        rawAnchorData: list[ShepherdAnchorT] = []
 
         if "dfns" in rawWSpec:
             currentAnchors = anchorsFromWebref("current", rawWSpec["dfns"])
             if currentAnchors:
-                rawAnchorData = [convertWebrefAnchor(x, currentUrl, "current") for x in currentAnchors]
+                for anchor in currentAnchors:
+                    addToAnchors(anchor, anchors, spec, "current")
         if "headings" in rawWSpec:
             currentHeadings = headingsFromWebref("current", rawWSpec["headings"])
             if currentHeadings:
-                rawAnchorData += [convertWebrefHeading(x, currentUrl, "current") for x in currentHeadings]
+                for heading in currentHeadings:
+                    addToHeadings(heading, specHeadings, spec, "current")
 
         # Complete list of anchors/headings with those from the snapshot version of the spec
         if spec["snapshot_url"] is not None and snapshotWebrefData is not None:
@@ -221,97 +188,13 @@ def gatherWebrefData(specs: SpecsT, anchors: AnchorsT, headings: AllHeadingsT) -
                 if "dfns" in rawSnapshotSpec:
                     snapshotAnchors = anchorsFromWebref("snapshot", rawSnapshotSpec["dfns"])
                     if snapshotAnchors:
-                        rawAnchorData += [
-                            convertWebrefAnchor(x, spec["snapshot_url"], "snapshot") for x in snapshotAnchors
-                        ]
+                        for anchor in snapshotAnchors:
+                            addToAnchors(anchor, anchors, spec, "snapshot")
                 if "headings" in rawSnapshotSpec:
                     snapshotHeadings = headingsFromWebref("snapshot", rawSnapshotSpec["headings"])
                     if snapshotHeadings:
-                        rawAnchorData += [
-                            convertWebrefHeading(x, spec["snapshot_url"], "snapshot") for x in snapshotHeadings
-                        ]
-
-        if len(rawAnchorData) > 0:
-            processRawAnchors(rawAnchorData, anchors, specHeadings, spec)
-
-
-def gatherShepherdData(specs: SpecsT, anchors: AnchorsT, headings: AllHeadingsT) -> None:
-    m.say("Downloading anchor data from Shepherd...")
-    shepherd = APIClient(
-        "https://api.csswg.org/shepherd/",
-        version="vnd.csswg.shepherd.v1",
-        ca_cert_path=certifi.where(),
-    )
-    rawSpecData = dataFromApi(shepherd, "specifications", draft=True)
-    if not rawSpecData:
-        return None
-
-    neededShepherdSpecs = [
-        "css-2022",
-        "css-color-3",
-        "css-color-6",
-        "css-color-hdr-1",
-        "css-conditional-values-1",
-        "css-contain-1",
-        "css-display-4",
-        "css-forms-1",
-        "css-grid-1",
-        "css-ui-3",
-        "css-writing-modes-3",
-        "dom-level-2-style",
-        "mediaqueries-3",
-    ]
-    lastMsgTime: float = 0
-    for i, rawSSpec in enumerate(rawSpecData.values(), 1):
-        lastMsgTime = config.doEvery(
-            s=5,
-            lastTime=lastMsgTime,
-            action=progressMessager(i, len(rawSpecData)),
-        )
-        if rawSSpec["name"] not in neededShepherdSpecs:
-            continue
-        if isSpecInList(genSpec(rawSSpec), specs):
-            # Gradually stop as they get filled in
-            continue
-        print(rawSSpec["name"])
-        rawSSpec = dataFromApi(shepherd, "specifications", draft=False, anchors=False, spec=rawSSpec["name"])
-        spec = genSpec(rawSSpec)
-        assert spec["vshortname"] is not None
-        specs[spec["vshortname"]] = spec
-        specHeadings: HeadingsT = {}
-        headings[spec["vshortname"]] = specHeadings
-
-        rawAnchorData = [setStatus(x, "snapshot") for x in linearizeAnchorTree(rawSSpec.get("anchors", []))] + [
-            setStatus(x, "current") for x in linearizeAnchorTree(rawSSpec.get("draft_anchors", []))
-        ]
-        processRawAnchors(rawAnchorData, anchors, specHeadings, spec)
-
-
-@tenacity.retry(reraise=True, stop=tenacity.stop_after_attempt(3), wait=tenacity.wait_random(1, 2))
-def dataFromApi(api: APIClient, *args: t.Any, **kwargs: t.Any) -> t.JSONT:
-    anchorDataContentTypes = [
-        "application/json",
-        "application/vnd.csswg.shepherd.v1+json",
-    ]
-    res = api.get(*args, **kwargs)
-    if not res:
-        raise Exception(
-            "Unknown error fetching anchor data. This might be transient; try again in a few minutes, and if it's still broken, please report it on GitHub."
-        )
-    data = res.data
-    if res.status_code == 406:
-        raise Exception(
-            "This version of the anchor-data API is no longer supported. Try updating Bikeshed. If the error persists, please report it on GitHub."
-        )
-    if res.content_type not in anchorDataContentTypes:
-        raise Exception(f"Unrecognized anchor-data content-type '{res.content_type}'.")
-    if res.status_code >= 300:
-        raise Exception(
-            f"Unknown error fetching anchor data; got status {res.status_code} and bytes:\n{data.decode('utf-8')}"
-        )
-    if isinstance(data, bytes):
-        raise Exception(f"Didn't get expected JSON data. Got:\n{data.decode('utf-8')}")
-    return t.cast("t.JSONT", data)
+                        for heading in snapshotHeadings:
+                            addToHeadings(heading, specHeadings, spec, "snapshot")
 
 
 def specsFromWebref(status: t.Literal["current" | "snapshot"]) -> list[WebrefSpecT]:
@@ -319,7 +202,13 @@ def specsFromWebref(status: t.Literal["current" | "snapshot"]) -> list[WebrefSpe
     j = dataFromWebref(url)
     if j is None or j.get("results") is None:
         raise Exception(f"No {status} specs data from WebRef. Got:\n{json.dumps(j, indent=1)}")
-    return t.cast("list[WebrefSpecT]", j["results"])
+    rawSpecs = t.cast("list[WebrefSpecT]", j["results"])
+    filteredSpecs: list[WebrefSpecT] = []
+    for spec in rawSpecs:
+        if spec["seriesComposition"] == "fork":
+            continue
+        filteredSpecs.append(spec)
+    return filteredSpecs
 
 
 def anchorsFromWebref(status: t.Literal["current" | "snapshot"], urlSuffix: str) -> list[WebrefAnchorT]:
@@ -354,57 +243,7 @@ def dataFromWebref(url: str) -> t.JSONT:
     return t.cast("t.JSONT", data)
 
 
-def linearizeAnchorTree(multiTree: list, rawAnchors: list[dict[str, t.Any]] | None = None) -> list[ShepherdAnchorT]:
-    if rawAnchors is None:
-        rawAnchors = []
-    # Call with multiTree being a list of trees
-    for item in multiTree:
-        if (item["type"] in config.dfnTypes.union(["dfn"])) or ("section" in item and item["section"] is True):
-            rawAnchors.append(item)
-        if item.get("children"):
-            linearizeAnchorTree(item["children"], rawAnchors)
-            del item["children"]
-    return t.cast("list[ShepherdAnchorT]", rawAnchors)
-
-
-def genSpec(rawSpec: ShepherdSpecT) -> SpecT:
-    assert rawSpec["name"] is not None
-    assert rawSpec["short_name"] is not None
-    assert rawSpec["title"] is not None
-    assert rawSpec["description"] is not None
-    spec: SpecT = {
-        "vshortname": rawSpec["name"],
-        "shortname": rawSpec["short_name"],
-        "snapshot_url": rawSpec.get("base_uri"),
-        "current_url": rawSpec.get("draft_uri"),
-        "title": rawSpec["title"],
-        "description": rawSpec["description"],
-        "abstract": rawSpec.get("abstract"),
-        "level": None,
-    }
-    vShortname = spec["vshortname"]
-    if spec["shortname"] is not None and vShortname.startswith(spec["shortname"]):
-        # S = "foo", V = "foo-3"
-        # Strip the prefix
-        level = vShortname[len(spec["shortname"]) :]
-        if level.startswith("-"):
-            level = level[1:]
-        if level.isdigit():
-            spec["level"] = int(level)
-        else:
-            spec["level"] = 1
-    elif spec["shortname"] is None and re.match(r"(.*)-(\d+)", vShortname):
-        # S = None, V = "foo-3"
-        match = t.cast(re.Match, re.match(r"(.*)-(\d+)", vShortname))
-        spec["shortname"] = match.group(1)
-        spec["level"] = int(match.group(2))
-    else:
-        spec["shortname"] = vShortname
-        spec["level"] = 1
-    return spec
-
-
-def genWebrefSpec(rawSpec: WebrefSpecT) -> SpecT:
+def canonSpecFromWebref(rawSpec: WebrefSpecT) -> SpecT:
     """Generate a spec object from data gleaned from Webref"""
     assert rawSpec["shortname"] is not None
     assert rawSpec["series"] is not None
@@ -426,127 +265,83 @@ def genWebrefSpec(rawSpec: WebrefSpecT) -> SpecT:
     return spec
 
 
-def convertWebrefHeading(heading: WebrefHeadingT, specUrl: str, status: str) -> ShepherdAnchorT:
-    """Convert a heading returned by Webref to the anchor format used in Shepherd"""
-    assert heading["id"] is not None
-    assert heading["title"] is not None
-    assert heading["href"] is not None
-    anchor: ShepherdAnchorT = {
-        "name": heading["number"] if "number" in heading else heading["id"],
-        "type": "heading",
-        "for": [],
-        "section": True,
-        "title": heading["title"],
-        "status": status,
-        "normative": True,
-        "export": True,
-        "linking_text": [heading["title"]],
-        "uri": heading["href"].replace(specUrl, ""),
-    }
-    return anchor
-
-
-def convertWebrefAnchor(rawAnchor: WebrefAnchorT, specUrl: str, status: str) -> ShepherdAnchorT:
-    """Convert an anchor returned by Webref to the anchor format used in Shepherd"""
-    assert rawAnchor["id"] is not None
+def addToAnchors(
+    rawAnchor: WebrefAnchorT, anchors: AnchorsT, spec: SpecT, status: t.Literal["current"] | t.Literal["snapshot"]
+) -> None:
+    baseUrl = spec["snapshot_url"] if status == "snapshot" else spec["current_url"]
+    assert baseUrl is not None
     assert rawAnchor["type"] is not None
     assert rawAnchor["linkingText"] is not None
     assert rawAnchor["access"] is not None
     assert rawAnchor["href"] is not None
-    anchor: ShepherdAnchorT = {
-        "name": rawAnchor["id"],
-        "type": rawAnchor["type"],
-        "for": rawAnchor["for"],
-        "section": False,
-        "title": rawAnchor["linkingText"][0],
+    assert rawAnchor["informative"] is not None
+    assert rawAnchor["for"] is not None
+
+    anchor: AnchorT = {
         "status": status,
-        "normative": not rawAnchor["informative"],
+        "type": rawAnchor["type"],
         "export": rawAnchor["access"] == "public",
-        "linking_text": rawAnchor["linkingText"],
-        "uri": rawAnchor["href"].replace(specUrl, ""),
+        "normative": not rawAnchor["informative"],
+        "url": rawAnchor["href"],
+        "for": rawAnchor["for"],
+        "spec": spec["vshortname"],
+        "shortname": spec["shortname"],
+        "level": int(spec["level"] or "1"),
     }
-    return anchor
+    for text in rawAnchor["linkingText"]:
+        text = re.sub(r"‘|’", "'", text)
+        text = re.sub(r"“|”", '"', text)
+        if anchor["type"] in config.lowercaseTypes:
+            text = text.lower()
+        text = re.sub(r"\s+", " ", text)
+        anchors[text].append(anchor)
 
 
-def fixupAnchor(anchor: ShepherdAnchorT) -> ShepherdAnchorT:
-    """Miscellaneous fixes to the anchors before I start processing"""
-
-    # This one issue was annoying
-    if anchor.get("title", None) == "'@import'":
-        anchor["title"] = "@import"
-
-    # css3-tables has this a bunch, for some strange reason
-    if (anchor.get("uri") or "").startswith("??"):
-        anchor["uri"] = t.cast(str, anchor.get("uri"))[2:]
-
-    # If any smart quotes crept in, replace them with ASCII.
-    linkingTexts: list[str] = anchor.get("linking_text") or [t.cast(str, anchor.get("title"))]
-    for i, text in enumerate(linkingTexts):
-        if text is None:
-            continue
-        if "’" in text or "‘" in text:
-            text = re.sub(r"‘|’", "'", text)
-            linkingTexts[i] = text
-        if "“" in text or "”" in text:
-            text = re.sub(r"“|”", '"', text)
-            linkingTexts[i] = text
-    anchor["linking_text"] = linkingTexts
-
-    # Normalize whitespace to a single space
-    def strip(s: str) -> str:
-        return re.sub(r"\s+", " ", s.strip())
-
-    anchor["name"] = strip(anchor["name"])
-    anchor["type"] = strip(anchor["type"])
-    anchor["for"] = [strip(x) for x in anchor.get("for", [])]
-    anchor["status"] = strip(anchor["status"])
-    anchor["linking_text"] = [strip(x) for x in anchor.get("linking_text", []) if x is not None]
-    anchor["uri"] = strip(anchor["uri"])
-
-    return anchor
-
-
-def addToHeadings(rawAnchor: ShepherdAnchorT, specHeadings: HeadingsT, spec: SpecT) -> None:
-    uri = rawAnchor["uri"]
-    assert uri is not None
-    if rawAnchor["status"] == "snapshot":
-        baseUrl = spec["snapshot_url"]
-    else:
-        baseUrl = spec["current_url"]
-    assert baseUrl is not None
+def addToHeadings(
+    rawAnchor: WebrefHeadingT,
+    specHeadings: HeadingsT,
+    spec: SpecT,
+    status: t.Literal["current"] | t.Literal["snapshot"],
+) -> None:
     heading: HeadingT = {
-        "url": baseUrl + uri,
-        "number": rawAnchor["name"] if re.match(r"[\d.]+$", rawAnchor["name"]) else "",
+        "url": rawAnchor["href"],
+        "number": rawAnchor["number"] if "number" in rawAnchor else "",
         "text": rawAnchor["title"],
         "spec": spec["title"],
     }
-    if uri[0] == "#":
+    specUrl = spec["snapshot_url"] if status == "snapshot" else spec["current_url"]
+    assert specUrl is not None
+    if heading["url"].startswith(specUrl):
+        truncatedUrl = heading["url"][len(specUrl) :]
+    else:
+        print(f"Invalid heading - URL doesn't start with the spec's url <{specUrl}>.\n{heading}")
+        return
+    if truncatedUrl[0] == "#":
         # Either single-page spec, or link on the top page of a multi-page spec
-        fragment = uri
+        fragment = truncatedUrl
         shorthand = "/" + fragment
     else:
         # Multi-page spec, need to guard against colliding IDs
-        if "#" in uri:
+        if "#" in truncatedUrl:
             # url to a heading in the page, like "foo.html#bar"
-            match = re.match(r"([\w-]+).*?(#.*)", uri)
+            match = re.match(r"([\w-]+).*?(#.*)", truncatedUrl)
             if not match:
                 m.die(
-                    f"Unexpected URI pattern '{uri}' for spec '{spec['vshortname']}'. Please report this to the Bikeshed maintainer.",
+                    f"Unexpected URI pattern '{truncatedUrl}' for spec '{spec['vshortname']}'. Please report this to the Bikeshed maintainer.",
                 )
                 return
             page, fragment = match.groups()
             page = "/" + page
         else:
             # url to a page itself, like "foo.html"
-            page, _, _ = uri.partition(".")
+            page, _, _ = truncatedUrl.partition(".")
             page = "/" + page
             fragment = "#"
         shorthand = page + fragment
     if shorthand not in specHeadings:
         specHeadings[shorthand] = {}
     headingGroup = t.cast("HeadingGroupT", specHeadings[shorthand])
-    headingGroup[rawAnchor["status"]] = heading
-    assert rawAnchor["status"] is not None
+    headingGroup[status] = heading
     if fragment not in specHeadings:
         specHeadings[fragment] = []
     keyList = t.cast("list[HeadingKeyT]", specHeadings[fragment])
@@ -571,49 +366,6 @@ def cleanSpecHeadings(headings: AllHeadingsT) -> None:
                 del specHeadings[v[0]]
 
 
-def addToAnchors(rawAnchor: ShepherdAnchorT, anchors: AnchorsT, spec: SpecT) -> None:
-    if rawAnchor["status"] == "snapshot":
-        baseUrl = spec["snapshot_url"]
-    else:
-        baseUrl = spec["current_url"]
-    assert baseUrl is not None
-    anchor: AnchorT = {
-        "status": rawAnchor["status"],
-        "type": rawAnchor["type"],
-        "spec": spec["vshortname"],
-        "shortname": spec["shortname"],
-        "level": int(spec["level"] or "1"),
-        "export": rawAnchor.get("export", False),
-        "normative": rawAnchor.get("normative", False),
-        "url": baseUrl + rawAnchor["uri"],
-        "for": rawAnchor.get("for", []),
-    }
-    for text in rawAnchor["linking_text"]:
-        if anchor["type"] in config.lowercaseTypes:
-            text = text.lower()
-        text = re.sub(r"\s+", " ", text)
-        anchors[text].append(anchor)
-
-
-def processRawAnchors(
-    rawAnchorData: list[ShepherdAnchorT], anchors: AnchorsT, specHeadings: HeadingsT, spec: SpecT
-) -> None:
-    for rawAnchor in rawAnchorData:
-        rawAnchor = fixupAnchor(rawAnchor)
-        linkingTexts = rawAnchor["linking_text"]
-        assert linkingTexts is not None
-        if len(linkingTexts) == 0:
-            # Happens if it had no linking text at all originally
-            continue
-        if len(linkingTexts) == 1 and linkingTexts[0].strip() == "":
-            # Happens if it was marked with an empty lt and Shepherd still picked it up
-            continue
-        if "section" in rawAnchor and rawAnchor["section"] is True:
-            addToHeadings(rawAnchor, specHeadings, spec=spec)
-        if rawAnchor["type"] in config.dfnTypes.union(["dfn"]):
-            addToAnchors(rawAnchor, anchors, spec=spec)
-
-
 def extractMethodData(anchors: AnchorsT) -> MethodsT:
     """Compile a db of {argless methods => {argfull method => {args, fors, url, shortname}}"""
 
@@ -660,23 +412,6 @@ def extractForsData(anchors: AnchorsT) -> ForsT:
     return fors
 
 
-def setStatus(obj: ShepherdAnchorT, status: str) -> ShepherdAnchorT:
-    obj["status"] = status
-    return obj
-
-
-def isSpecInList(spec: SpecT, specs: SpecsT) -> bool:
-    if spec["vshortname"].lower() in specs:
-        return True
-    for s in specs.values():
-        # Shortnames don't always match. Let's compare URLs
-        if (s["snapshot_url"] is not None and s["snapshot_url"] == spec["snapshot_url"]) or (
-            s["current_url"] is not None and s["current_url"] == spec["current_url"]
-        ):
-            return True
-    return False
-
-
 def writeAnchorsFile(anchors: AnchorsT, path: str) -> set[str]:
     """
     Keys may be duplicated.
